@@ -5,7 +5,7 @@ import numpy as np
 
 from .known import vlr_factory, IKnownVLR
 from .vlr import VLR
-from ..utils import encode_to_len
+from ..utils import encode_to_len, read_string, write_as_c_string
 
 logger = logging.getLogger(__name__)
 
@@ -63,18 +63,12 @@ class VLRList(list):
             a list of vlrs matching the user_id and records_ids
 
         """
-        if user_id != "" and record_ids != (None,):
-            return [
-                vlr
-                for vlr in self
-                if vlr.user_id == user_id and vlr.record_id in record_ids
-            ]
-        else:
-            return [
-                vlr
-                for vlr in self
-                if vlr.user_id == user_id or vlr.record_id in record_ids
-            ]
+        return [
+            vlr
+            for vlr in self
+            if (user_id == "" or vlr.user_id == user_id)
+            and (record_ids == (None,) or vlr.record_id in record_ids)
+        ]
 
     def get(self, vlr_type: str) -> List[IKnownVLR]:
         """Returns the list of vlrs of the requested type
@@ -173,7 +167,7 @@ class VLRList(list):
                 record_data_len = int.from_bytes(
                     data_stream.read(2), byteorder="little", signed=False
                 )
-            description = data_stream.read(DESCRIPTION_LEN).split(b"\0")[0].decode()
+            description = read_string(data_stream, DESCRIPTION_LEN)
             record_data_bytes = data_stream.read(record_data_len)
 
             vlr = VLR(user_id, record_id, description, record_data_bytes)
@@ -182,25 +176,35 @@ class VLRList(list):
 
         return vlrlist
 
-    def write_to(self, stream: BinaryIO, as_extended: bool = False) -> int:
+    def write_to(
+        self,
+        stream: BinaryIO,
+        as_extended: bool = False,
+        encoding_errors: str = "strict",
+    ) -> int:
         bytes_written = 0
         for vlr in self:
             record_data = vlr.record_data_bytes()
 
             stream.write(b"\0\0")
-            stream.write(encode_to_len(vlr.user_id, USER_ID_LEN))
+            write_as_c_string(stream, vlr.user_id, USER_ID_LEN)
             stream.write(vlr.record_id.to_bytes(2, byteorder="little", signed=False))
             if as_extended:
-                if len(record_data) > np.iinfo("uint16").max:
-                    raise ValueError("vlr record_data is too long")
                 stream.write(
                     len(record_data).to_bytes(8, byteorder="little", signed=False)
                 )
             else:
+                if len(record_data) > np.iinfo("uint16").max:
+                    raise ValueError("vlr record_data is too long")
                 stream.write(
                     len(record_data).to_bytes(2, byteorder="little", signed=False)
                 )
-            stream.write(encode_to_len(vlr.description, DESCRIPTION_LEN))
+            write_as_c_string(
+                stream,
+                vlr.description,
+                DESCRIPTION_LEN,
+                encoding_errors=encoding_errors,
+            )
             stream.write(record_data)
 
             bytes_written += 54 if not as_extended else 60
